<?php

namespace Drupal\commerce_stripe_payment_request_button\Controller;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\Payment;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;
use Drupal\profile\Entity\Profile;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

class PaymentRequestButtonCheckout {

  public function handleOrder(Request $request) {
    // Transform JSON values to array
    $values = Json::decode($request->getContent());
    $order_id = $values["data"]["order_id"];

    $carts = \Drupal::getContainer()
      ->get('commerce_cart.cart_provider')
      ->getCarts();
    $active_cart = array_filter($carts, function (OrderInterface $cart) use ($order_id) {
      return $cart->id() == $order_id;
    });

    ///TODO use $values["data"]["result"]["shippingOption"] to assigne proper shipping method after user has chosen it

    // Form values are passed as a serialized string.
    $form_values = [];
    parse_str($values["data"]["serialized_form"], $form_values);

    // Init Stripe.
    $config = \Drupal::config('commerce_payment.commerce_payment_gateway.commerce_stripe_payment_request_button')
      ->get('configuration');
    \Stripe\Stripe::setApiKey($config['secret_key']);
    $active_cart = reset($active_cart);

    // @see \Drupal\commerce_checkout\Controller\CheckoutController::checkAccess
    if (empty($active_cart) || $active_cart->getState()->value == 'canceled') {
      $response = [
        'redirect_uri' => Url::fromRoute(Url::fromRoute('commerce_cart.page')
          ->toString()),
      ];
      return new JsonResponse($response);
    }

    $description = $values["data"]["result"]["token"]["card"]["id"] . ', ' . \Drupal::config('system.site')
        ->get('name');
    try {
      // Charge the user's card immediately.
      $charge = \Stripe\Charge::create([
        'amount' => $values["data"]["payment_request"]["total"]["amount"],
        'currency' => $values["data"]["payment_request"]["currency"],
        'description' => $description,
        'metadata' => [
          "order_id" => $values["data"]["order_id"],
          "email" => $values["data"]["result"]["payerEmail"],
        ],
        'source' => $values["data"]["token"],
      ]);
    } catch (\Stripe\Error\Base $e) {
      watchdog_exception('commerce_stripe_payment_request', $e);
      // Server will return 500 to the frontend.
      \Drupal::messenger()->addError($e->getMessage());
    }


    if ($active_cart->hasItems()) {
      // Put order in proper status and assign values
      $active_cart->get('checkout_flow')->setValue(NULL);
      $active_cart->get('checkout_step')->setValue("payment");
      $active_cart->setEmail($values["data"]["result"]["payerEmail"]);
      $active_cart->get('payment_gateway')
        ->setValue("commerce_stripe_payment_request_button");
      $active_cart->save();

      // Create payment for this order and give it starting values
      $payment_gateway = \Drupal::entityTypeManager()
        ->getStorage('commerce_payment_gateway')
        ->load('commerce_stripe_payment_request_button');

      $current_user = $active_cart->getCustomerId();

      $payment = Payment::create([
        'state' => 'new',
        'amount' => $active_cart->getTotalPrice(),
        'payment_gateway' => $payment_gateway->id(),
        'order_id' => $active_cart->id(),
        'remote_id' => $values["data"]["result"]["token"]["card"]["id"],
        'payment_gateway_mode' => $payment_gateway->getPlugin()->getMode(),
        'expires' => 0,
        'uid' => $current_user,
      ]);
      $payment->save();

      // Deal with billing profile
      $profile = $active_cart->getBillingProfile();

      if (empty($profile)) {
        $profile = Profile::create([
          'type' => 'customer',
          'uid' => $current_user,
        ]);
        $profile->save();
      }

      $current_name =  isset($values["data"]["result"]["payerName"])?:$values["data"]["result"]["token"]["card"]["name"];
      $names = explode(' ', $current_name);
      $given_name = array_shift($names);
      $family_name = implode(' ', $names);

      $profile->address->given_name = $given_name;
      $profile->address->family_name = $family_name;
      $profile->address->country_code = $values["data"]["result"]["token"]["card"]["country"];
      $profile->address->locality = $values["data"]["result"]["token"]["card"]["address_city"];
      $profile->address->administrative_area = $values["data"]["result"]["token"]["card"]["address_state"];
      $profile->address->postal_code = $values["data"]["result"]["token"]["card"]["address_zip"];
      $profile->address->address_line1 = $values["data"]["result"]["token"]["card"]["address_line1"];
      $profile->address->address_line2 = $values["data"]["result"]["token"]["card"]["address_line2"];
      $profile->save();

      $active_cart->setBillingProfile($profile);
      $active_cart->save();
    }

    // Gets url from which payment step is set and sends it to JS
    $response = [
      'redirect_uri' => Url::fromRoute('commerce_checkout.form', [
        'commerce_order' => $active_cart->id(),
      ])->toString(),
    ];


    return new JsonResponse($response);
  }


}
